package org.diagbot.common.push.filter.rule;


import org.diagbot.common.push.bean.PreResult;
import org.diagbot.common.push.bean.ResponseData;
import org.diagbot.common.push.bean.SearchData;
import org.diagbot.common.push.filter.pretreat.*;
import org.diagbot.pub.Constants;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class PretreatmentRule {
    public void rule(SearchData searchData) throws java.io.IOException {
        //症状数据
        if (!StringUtils.isEmpty(searchData.getSymptom())) {
            searchData.setSymptom(add2PreResultList(new PretreatmentSymptom(), searchData.getSymptom(), "symptom"));
        }
        //体征数据
        if (!StringUtils.isEmpty(searchData.getVital())) {
            searchData.setVital(add2PreResultList(new PretreatmentVital(), searchData.getVital(), "vital"));
        }
        //lis结构化信息
        if (searchData.getLisArr() != null && searchData.getLisArr().size() > 0) {
            searchData.setLis(add2PreResultList(searchData.getLisArr(), searchData.getLis(), "lis"));
        } else if (!StringUtils.isEmpty(searchData.getLis())) {
            searchData.setLis(add2PreResultList(new PretreatmentLis(), searchData.getLis(), "lis"));
        }
        //pacs数据
        if (!StringUtils.isEmpty(searchData.getPacs())) {
            searchData.setPacs(add2PreResultList(new PretreatmentPacs(), searchData.getPacs(), "pacs"));
        }
    }

    private String add2PreResultList(Pretreatment pretreatment, String content, String ruleType) throws java.io.IOException {
        List<PreResult> preResultList = pretreatment.analyze(content);
        return add2PreResultList(preResultList, content, ruleType);
    }

    private String add2PreResultList(List<PreResult> preResultList, String content, String ruleType) throws java.io.IOException {
        Map<String, List<Rule>> kl_rule_filter_map = ApplicationCacheUtil.getKl_rule_filter_map();
        if (preResultList != null) {
            for (PreResult result : preResultList) {
                //规则库中匹配
                if (kl_rule_filter_map.get(result.getDetailName()) != null) {
                    List<Rule> rules = kl_rule_filter_map.get(result.getDetailName());
                    if (rules == null) {
                        continue;
                    }
                    for (Rule rule : rules) {
                        if (Constants.COMMON_STRING_1.equals(rule.getSet_status())) {   //需要匹配大类
                            if (StringUtils.isEmpty(rule.getSet_name()) || !result.getName().equals(rule.getSet_name())) {
                                continue;
                            }
                        }
                        content = conceptConvert(result, rule, content);
                        //危急值提醒
                        if (Constants.COMMON_STRING_1.equals(rule.getType_value())) {
                            conceptCrisis(result, rule, allCrisisDetailList);
                        }
                    }
                }
            }

        }
        return content;
    }

    private String conceptConvert(PreResult result, Rule rule, String content) {
        //标准值最优先匹配 暂时忽略单位
        if (compareEqual(result.getValue(), rule.getSet_value())) {
            content = content + rule.getSet_concept_text()==null?"":rule.getSet_concept_text();
        }
        if (compareMin(result.getValue(), rule.getMin_value())) {
            content = content + rule.getMin_concept_text()==null?"":rule.getMin_concept_text();
        }
        if (compareMax(result.getValue(), rule.getMax_value())) {
            content = content + rule.getMax_concept_text()==null?"":rule.getMax_concept_text();
        }
        return content;
    }

    private void conceptCrisis(PreResult result, Rule rule, List<CrisisDetail> allCrisisDetailList) {
        boolean crisis = false;
        CrisisDetail crisisDetail = new CrisisDetail();
        crisisDetail.setOriginText(result.getDetailName() + " : " + result.getValue() + result.getUnits());
        if (compareEqual(result.getValue(), rule.getSet_value())) {
            crisisDetail.setStandardText(result.getDetailName() + " : " + rule.getSet_value() + rule.getUnit());
            crisisDetail.setRemindText(rule.getSetRemind());
            crisis = true;
        }
        if (compareMin(result.getValue(), rule.getMin_value())) {
            crisisDetail.setStandardText(result.getDetailName() + " <= " + rule.getMin_value() + rule.getUnit());
            crisisDetail.setRemindText(rule.getMinRemind());
            crisis = true;
        }
        if (compareMax(result.getValue(), rule.getMax_value())) {
            crisisDetail.setStandardText(result.getDetailName() + " >= " + rule.getMax_value() + rule.getUnit());
            crisisDetail.setRemindText(rule.getMaxRemind());
            crisis = true;
        }
        if (crisis) {
            allCrisisDetailList.add(crisisDetail);
        }
    }

    private boolean compareEqual(String c1, String c2) {
        if (!StringUtils.isEmpty(c1) && !StringUtils.isEmpty(c2)
                && c1.equals(c2)) {
            return true;
        }
        return false;
    }

    private boolean compareMin(String c1, String c2) {
        if (!StringUtils.isEmpty(c1) && !StringUtils.isEmpty(c2)) {
            try {
                return Double.valueOf(c1) < Double.valueOf(c2);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return false;
    }

    private boolean compareMax(String c1, String c2) {
        if (!StringUtils.isEmpty(c1) && !StringUtils.isEmpty(c2)) {
            try {
                return Double.valueOf(c1) > Double.valueOf(c2);
            } catch (Exception e) {}
        }
        return false;
    }
}
